@@ -1,6 +1,6 @@
 import json
 import logging
-
+import re
 import boto3
 import fnmatch
 import os
@@ -19,7 +19,10 @@ class Catalog:
         self,
         repo: BaseRepo,
         environment: HelixEnvironment = HelixEnvironment.PRODUCTION,
-    ):
+    ) -> None:
+        """
+        Implementation of the data catalog
+        """
         self.logger = logging.getLogger(self.__class__.__name__)
         self.logger.setLevel(os.environ.get("LOGLEVEL", "INFO"))
         self._repo: BaseRepo = repo
@@ -69,6 +72,62 @@ class Catalog:
                 )
                 self.update_data_source(data_source.name, data_source.to_json())
 
+    def update_resource_last_processed_date_segment(
+        self,
+        resource_name: str,
+        updated_date: datetime,
+        data_source: DataSource,
+        environment: HelixEnvironment,
+    ) -> None:
+        """
+        Updates the last_processed path with the passed in date using the date_format of the resource
+        :param resource_name: resource name to update
+        :param updated_date: date to update last_processed with
+        :param data_source: data source to update
+        :param environment: environment
+        """
+        resource: ResourceItem
+        for resource in data_source.resources:
+            if resource_name == resource.name and resource.last_processed is not None:
+                date_segment: Union[int, List[int]] = (
+                    resource.date_segment if resource.date_segment else 0
+                )
+                self.logger.info(f"Using date_segment: {date_segment}")
+                date_format: str = (
+                    resource.date_format if resource.date_format else "%Y-%m-%d"
+                )
+                last_processed_path: str = resource.last_processed.get_last_processed(
+                    environment
+                )
+                last_processed_path_parts: List[str] = list(
+                    filter(None, last_processed_path.replace("s3:/", "").split("/"))
+                )
+                if isinstance(date_segment, int):
+                    # replace that part of the path with new one
+                    last_processed_path_parts[date_segment] = updated_date.strftime(
+                        date_format
+                    )
+                elif isinstance(date_segment, list):
+                    date_format_parts = re.split(r"(%)", date_format)
+                    for segment, date_format_part in zip(
+                        date_segment, date_format_parts
+                    ):
+                        last_processed_path_parts[segment] = updated_date.strftime(
+                            date_format_part
+                        )
+
+                new_last_processed_path: str = "/".join(last_processed_path_parts)
+
+                self.logger.info(
+                    f"Updating last_processed from {last_processed_path} to {new_last_processed_path}"
+                    f" for environment {environment}"
+                )
+
+                resource.last_processed.set_last_processed(
+                    new_last_processed_path, environment
+                )
+                self.update_data_source(data_source.name, data_source.to_json())
+
     def update_data_source(self, data_source_name: str, updated_contents: str) -> None:
         self._repo.update_file(data_source_name, updated_contents)
 
@@ -89,15 +148,17 @@ class Catalog:
             date_format: str = (
                 resource.date_format if resource.date_format else "%Y-%m-%d"
             )
-            last_processed: str = resource.last_processed.get_last_processed(
+            last_processed_path: str = resource.last_processed.get_last_processed(
                 environment
             )
             self.logger.info(
-                f"Found last processed path {last_processed} for resource {resource.name}"
+                f"Found last processed path {last_processed_path} for resource {resource.name}"
+            )
+            all_paths: List[str] = self.get_all_directories(
+                resource, last_processed_path
             )
-            all_paths: List[str] = self.get_all_directories(resource, last_processed)
             self.logger.debug(
-                f"---- Found directories for path [{last_processed}] ------"
+                f"---- Found paths for last processed path [{last_processed_path}] ------"
             )
             for path in all_paths:
                 self.logger.debug(f"{path}")
@@ -105,7 +166,7 @@ class Catalog:
                 "-----------------------------------------------------------"
             )
             path_parts: List[str] = list(
-                filter(None, last_processed.replace("s3:/", "").split("/"))
+                filter(None, last_processed_path.replace("s3:/", "").split("/"))
             )
 
             last_processed_date: Optional[datetime] = None
@@ -153,7 +214,7 @@ class Catalog:
                 sorted_paths_to_process.append(paths_to_process[path_date])
 
             self.logger.info(
-                f"---- Sorted directories for path [{last_processed}] ------"
+                f"---- Sorted paths for last processed path [{last_processed_path}] ------"
             )
             for path in sorted_paths_to_process:
                 self.logger.info(f"{path}")
